@@ -1,6 +1,6 @@
 # IXIN.pm: output IXIN format.
 #
-# Copyright 2013-2024 Free Software Foundation, Inc.
+# Copyright 2013-2026 Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Original author: Patrice Dumas <pertusus@free.fr>
 #
@@ -31,7 +31,7 @@
 # that does so.  A module inheriting both from a converter module, for
 # convert_tree() and convert_output_unit(), and this module
 # should be used.  A functional implementation of IXIN is available as the
-# Texinfo::Convert::IXINSXML module which uses Texinfo::Convert::TexinfoSXML
+# Texinfo::Example::IXINSXML module which uses Texinfo::Convert::TexinfoSXML
 # for the Texinfo tree conversion.  Using a Texinfo tree converter that does
 # not lose information for Texinfo tree conversion in IXIN is in line with the
 # IXIN philosophy, so conversion modules like Texinfo::Convert::TexinfoXML or
@@ -61,7 +61,7 @@
 # and
 #  https://lists.gnu.org/archive/html/help-texinfo/2015-02/msg00004.html
 
-package Texinfo::Convert::IXIN;
+package Texinfo::Example::IXIN;
 
 use 5.006;
 use strict;
@@ -84,7 +84,7 @@ use Texinfo::Convert::TexinfoSXML;
 
 our @ISA = qw(Texinfo::Convert::Converter);
 
-our $VERSION = '7.2';
+our $VERSION = '7.3';
 
 
 my $ixin_version = 1;
@@ -100,9 +100,11 @@ my %additional_setting_commands;
 # but are considered to be settings.
 # FIXME some seem to be missing, 'smallbook', 'afourpaper', 'afivepaper',
 # 'afourlatex', 'afourwide', 'bsixpaper'
+# 'codequoteundirected', 'codequotebacktick'.  If selected, set and clear
+# should be too as synonyms with txicode* flags.
 foreach my $command ('pagesizes', 'everyheading', 'everyfooting',
                      'evenheading', 'evenfooting', 'oddheading', 'oddfooting',
-                     'documentencoding', 'documentlanguage', 'clickstyle') {
+                     'documentencoding', 'documentlanguage') {
   $additional_setting_commands{$command} = 1;
 }
 
@@ -239,8 +241,7 @@ sub ixin_none_element($$)
 # which is always up to date and handles better content in @insertcopying
 # or @titlepage, but has specific HTML code related to separate
 # elements.
-sub _get_element($$);
-sub _get_element($$)
+sub _get_root_command_output_unit($$)
 {
   my $self = shift;
   my $current = shift;
@@ -268,7 +269,8 @@ sub _count_bytes($$)
   my $self = shift;
   my $string = shift;
 
-  return Texinfo::Common::count_bytes($self, $string);
+  return Texinfo::Common::count_bytes($string,
+                               $self->get_conf('OUTPUT_ENCODING_NAME'));
 }
 
 sub _associated_node_id($$$;$)
@@ -279,15 +281,14 @@ sub _associated_node_id($$$;$)
   my $node_command = shift;
 
   if (!defined($node_command)) {
-    my ($element, $root_command) = $self->_get_element($command);
+    my ($output_unit, $root_command)
+      = $self->_get_root_command_output_unit($command);
 
     if ($root_command) {
       if (!$root_command->{'cmdname'} or $root_command->{'cmdname'} ne 'node') {
-        if ($element
-            and $element->{'unit_command'}
-            and $element->{'unit_command'}->{'cmdname'}
-            and $element->{'unit_command'}->{'cmdname'} eq 'node') {
-          $node_command = $element->{'unit_command'};
+        if ($output_unit
+            and $output_unit->{'unit_node'}) {
+          $node_command = $output_unit->{'unit_node'}->{'element'};
         }
       } else {
         $node_command = $root_command;
@@ -341,7 +342,7 @@ sub output_ixin($$)
 
   my $fh;
   my $encoded_output_file;
-  if (! $output_file eq '') {
+  if ($output_file ne '') {
     my $path_encoding;
     ($encoded_output_file, $path_encoding)
       = $self->encoded_output_file_name($output_file);
@@ -349,8 +350,9 @@ sub output_ixin($$)
     # the third return information, set if the file has already been used
     # in this files_information is not checked as this cannot happen.
     ($fh, $error_message) = Texinfo::Convert::Utils::output_files_open_out(
-                             $self->output_files_information(), $self,
-                             $encoded_output_file);
+                             $self->output_files_information(),
+                             $encoded_output_file,
+                             undef, $self->get_conf('OUTPUT_ENCODING_NAME'));
     if (!$fh) {
       $self->converter_document_error(
                 sprintf(__("could not open %s for writing: %s"),
@@ -371,14 +373,11 @@ sub output_ixin($$)
                            [['name', $output_file_name]]);
   }
   my $lang = $self->get_conf('documentlanguage');
-  #my $lang_code = $lang;
-  #my $region_code;
-  #if ($lang =~ /^([a-z]+)_([A-Z]+)/) {
-  #  $lang_code = $1;
-  #  $region_code = $2;
-  #}
-  $result .= ' ';
-  $result .= $self->ixin_list_element('lang', [['name', $lang]]);
+  if (defined($lang)) {
+    $result .= ' ';
+    $result .= $self->ixin_list_element('lang', [['name', $lang]]);
+  }
+
   # FIXME title: use simpletitle or fulltitle
 
   my $document_info;
@@ -387,6 +386,7 @@ sub output_ixin($$)
   my $identifiers_target;
   my $indices_information;
   my $global_commands;
+  my $sectioning_root;
   if ($self->{'document'}) {
     $document_info = $self->{'document'}->global_information();
     $floats = $self->{'document'}->floats_information();
@@ -394,6 +394,7 @@ sub output_ixin($$)
     $identifiers_target = $self->{'document'}->labels_information();
     $indices_information = $self->{'document'}->indices_information();
     $global_commands = $self->{'document'}->global_commands_information();
+    $sectioning_root = $self->{'document'}->sectioning_root();
   }
 
   if ($global_commands and $global_commands->{'dircategory_direntry'}) {
@@ -442,7 +443,8 @@ sub output_ixin($$)
             = $Texinfo::Options::multiple_at_command_options{$global_command};
         }
         foreach my $command (@{$global_commands->{$global_command}}) {
-          my ($element, $root_command) = _get_element($self, $command);
+          my ($output_unit, $root_command)
+            = $self->_get_root_command_output_unit($command);
           # before first node
           if (not $root_command
               or not defined($root_command->{'extra'})
@@ -463,9 +465,8 @@ sub output_ixin($$)
   my %settings;
   foreach my $setting_command_name (keys(%setting_commands)) {
     my $setting_command = $setting_commands{$setting_command_name};
-    $setting_command_name = 'shortcontents'
-        if ($setting_command_name eq 'summarycontents');
-    my $value = Texinfo::Common::informative_command_value($setting_command);
+    my ($setting_command_name, $value)
+      = Texinfo::Common::informative_command_value($setting_command);
     #print STDERR "$setting_command_name $value\n";
     # do not register settings if set at the default value.
     if (defined($value)
@@ -541,7 +542,8 @@ sub output_ixin($$)
     if (defined($end_of_nodes_setting_commands{$normalized_node_name})) {
       foreach my $setting_command_name (keys(%{$end_of_nodes_setting_commands{$normalized_node_name}})) {
         # FIXME should use get_conf instead?
-        my $value = Texinfo::Common::_informative_command_value_informative_command_value(
+        # FIXME does not exist anymore
+        my $value = Texinfo::Common::_informative_command_value(
           $end_of_nodes_setting_commands{$normalized_node_name}->{$setting_command_name});
         if ((defined($settings{$setting_command_name})
              and $settings{$setting_command_name} eq $value)
@@ -557,17 +559,19 @@ sub output_ixin($$)
   }
 
   my $nodes_index = $self->ixin_open_element('nodesindex');
+  my $nodes_list = $self->{'document'}->nodes_list();
   foreach my $node (@nodes) {
     my $normalized_node_name = $node->{'extra'}->{'normalized'};
     # FIXME name should be a renderable sequence
     my @attributes = (['name', $normalized_node_name],
                       ['length',
                        $output_unit_byte_sizes{$normalized_node_name}]);
+    my $node_relations = $nodes_list->[$node->{'extra'}->{'node_number'} -1];
     foreach my $direction (@node_directions) {
-      if ($node->{'extra'}->{'node_directions'}
-          and $node->{'extra'}->{'node_directions'}->{lc($direction)}) {
+      if ($node_relations->{'node_directions'}
+          and $node_relations->{'node_directions'}->{lc($direction)}) {
         my $node_direction
-           = $node->{'extra'}->{'node_directions'}->{lc($direction)};
+           = $node_relations->{'node_directions'}->{lc($direction)};
         if ($node_direction->{'extra'}->{'manual_content'}) {
           # FIXME?
           push @attributes, ['node'.lc($direction), -2];
@@ -609,12 +613,12 @@ sub output_ixin($$)
   my $sectioning_tree = '';
   $sectioning_tree  .= $self->ixin_open_element('sectioningtree');
   if ($sections_list) {
-    my $section_root = $sections_list->[0]
-                                   ->{'extra'}->{'sectioning_root'};
-    foreach my $top_section (@{$section_root->{'extra'}->{'section_childs'}}) {
-      my $section = $top_section;
+    foreach my $top_relations (@{$sectioning_root->{'section_children'}}) {
+      my $section_relations = $top_relations;
  SECTION:
-      while ($section) {
+      while ($section_relations) {
+        my $section = $section_relations->{'element'};
+
         my $associated_node_id = $self->_associated_node_id($section,
                                                      \%node_label_number);
         my @attributes = (['nodeid', $associated_node_id], ['type',
@@ -622,8 +626,10 @@ sub output_ixin($$)
         $sectioning_tree .= $self->ixin_open_element('sectionentry',
                  \@attributes);
         $sectioning_tree .= $self->ixin_open_element('sectiontitle');
-        if ($section->{'args'} and $section->{'args'}->[0]) {
-          $sectioning_tree .= $self->convert_tree($section->{'args'}->[0]);
+        if ($section->{'contents'}->[0]->{'contents'}->[0]) {
+          $sectioning_tree
+            .= $self->convert_tree(
+                 $section->{'contents'}->[0]->{'contents'}->[0]);
         }
         $sectioning_tree .= $self->ixin_close_element('sectiontitle');
         # top is special and never considered to contain anything.  So
@@ -631,32 +637,35 @@ sub output_ixin($$)
         if ($section->{'cmdname'} eq 'top') {
           $sectioning_tree .= $self->ixin_close_element('sectionentry');
         }
-        if ($section->{'extra'}->{'section_childs'}) {
-          $section = $section->{'extra'}->{'section_childs'}->[0];
-        } elsif ($section->{'extra'}->{'section_directions'}
-                 and $section->{'extra'}->{'section_directions'}->{'next'}) {
+        if ($section_relations->{'section_children'}) {
+          $section_relations = $section_relations->{'section_children'}->[0];
+        } elsif ($section_relations->{'section_directions'}
+                 and $section_relations->{'section_directions'}->{'next'}) {
           $sectioning_tree .= $self->ixin_close_element('sectionentry');
-          last if ($section eq $top_section);
-          $section = $section->{'extra'}->{'section_directions'}->{'next'};
+          last if ($section_relations eq $top_relations);
+          $section_relations
+            = $section_relations->{'section_directions'}->{'next'};
         } else {
-          if ($section eq $top_section) {
+          if ($section_relations eq $top_relations) {
             $sectioning_tree .= $self->ixin_close_element('sectionentry')
               unless ($section->{'cmdname'} eq 'top');
             last;
           }
-          while ($section->{'extra'}->{'section_directions'}
-                 and $section->{'extra'}->{'section_directions'}->{'up'}) {
-            $section = $section->{'extra'}->{'section_directions'}->{'up'};
+          while ($section_relations->{'section_directions'}
+                 and $section_relations->{'section_directions'}->{'up'}) {
+            $section_relations
+              = $section_relations->{'section_directions'}->{'up'};
             $sectioning_tree .= $self->ixin_close_element('sectionentry');
-            if ($section eq $top_section) {
+            if ($section_relations eq $top_relations) {
               $sectioning_tree .= $self->ixin_close_element('sectionentry')
                  unless ($section->{'cmdname'} eq 'top');
               last SECTION;
             }
-            if ($section->{'extra'}->{'section_directions'}
-                and $section->{'extra'}->{'section_directions'}->{'next'}) {
+            if ($section_relations->{'section_directions'}
+                and $section_relations->{'section_directions'}->{'next'}) {
               $sectioning_tree .= $self->ixin_close_element('sectionentry');
-              $section = $section->{'extra'}->{'section_directions'}->{'next'};
+              $section_relations
+                = $section_relations->{'section_directions'}->{'next'};
               last;
             }
           }
@@ -713,14 +722,8 @@ sub output_ixin($$)
       my $dts_in_code = $indices_information->{$index_name}->{'in_code'};
       foreach my $dts_entry (@{$entries->{$index_name}}) {
         my $main_entry_element = $dts_entry->{'entry_element'};
-        my $node = $main_entry_element->{'extra'}->{'element_node'};
-        my $associated_node_id;
-        if (defined($node)) {
-          $associated_node_id = $self->_associated_node_id(undef,
-                                                    \%node_label_number, $node);
-        } else {
-          $associated_node_id = -1;
-        }
+        my $associated_node_id = $main_entry_element->{'extra'}->{'element_node'};
+        $associated_node_id = -1 if (!defined($associated_node_id));
         my $entry_content_element
           = Texinfo::Common::index_content_element($main_entry_element);
         my $entry = $self->convert_tree($entry_content_element);
@@ -803,7 +806,7 @@ sub output_ixin($$)
       my $float_type = $listoffloats_element->{'extra'}->{'float_type'};
       if ($float_type ne '') {
         $floats_information{$float_type}->{'type'}
-          = $self->convert_tree($listoffloats_element->{'args'}->[0]);
+          = $self->convert_tree($listoffloats_element->{'contents'}->[0]);
       }
       push @{$floats_information{$float_type}->{'node_id'}}, $associated_node_id;
     }
@@ -817,7 +820,8 @@ sub output_ixin($$)
     if ($floats->{$type}) {
       my $float_nr = 0;
       my $float_text = '';
-      foreach my $float (@{$floats->{$type}}) {
+      foreach my $float_and_section (@{$floats->{$type}}) {
+        my ($float, $float_section) = @$float_and_section;
         $float_nr++;
         my $associated_node_id;
         # associated node already found when collecting labels
@@ -836,19 +840,29 @@ sub output_ixin($$)
         } else {
           $float_text .= $self->ixin_none_element('floatlabel');
         }
-        if ($float->{'args'} and scalar(@{$float->{'args'}}) >= 2
-            and $float->{'args'}->[1]->{'contents'}) {
+        my $argument = $float->{'contents'}->[0];
+        if ($argument->{'contents'} and scalar(@{$argument->{'contents'}}) >= 2
+            and $argument->{'contents'}->[1]->{'contents'}) {
           $float_text .= $self->ixin_open_element('floatname');
           $float_text .= $self->convert_tree({'contents'
-                                 => $float->{'args'}->[1]->{'contents'}});
+                          => $argument->{'contents'}->[1]->{'contents'}});
           $float_text .= $self->ixin_close_element('floatname');
         } else {
           $float_text .= $self->ixin_none_element('floatname');
         }
-        if ($float->{'extra'}->{'shortcaption'}) {
-          $float_text .= $self->convert_tree($float->{'extra'}->{'shortcaption'});
-        } elsif ($float->{'extra'}->{'caption'}) {
-          $float_text .= $self->convert_tree($float->{'extra'}->{'caption'});
+
+        my ($caption, $shortcaption)
+          = Texinfo::Common::find_float_caption_shortcaption($float);
+
+        my $caption_element;
+        if ($shortcaption) {
+          $caption_element = $shortcaption;
+        } elsif ($caption) {
+          $caption_element = $caption;
+        }
+
+        if ($caption_element) {
+          $float_text .= $self->convert_tree($caption_element);
         } else {
           $float_text .= $self->ixin_none_element('caption');
         }
@@ -862,10 +876,12 @@ sub output_ixin($$)
       # determine type expandable string from first float if it was not
       # already determined from listoffloats
       if (!defined($floats_information{$type}->{'type'})) {
-        my $float_element = $floats->{$type}->[0];
+        my $float_and_section = $floats->{$type}->[0];
+        my ($float_element, $float_section) = @$float_and_section;
         if ($float_element->{'extra'}->{'float_type'} ne '') {
+          my $argument = $float_element->{'contents'}->[0];
           $floats_information{$type}->{'type'}
-            = $self->convert_tree($float_element->{'args'}->[0]);
+            = $self->convert_tree($argument->{'contents'}->[0]);
         }
       }
     }
@@ -912,15 +928,15 @@ sub output_ixin($$)
       my @extension;
       my $basefile;
       my $extension;
-      if (defined($command->{'args'}->[0])
-          and $command->{'args'}->[0]->{'contents'}
-          and @{$command->{'args'}->[0]->{'contents'}}) {
+      if (defined($command->{'contents'}->[0])
+          and $command->{'contents'}->[0]->{'contents'}
+          and @{$command->{'contents'}->[0]->{'contents'}}) {
         Texinfo::Convert::Text::set_options_code(
                                  $self->{'convert_text_options'});
         Texinfo::Convert::Text::set_options_encoding_if_not_ascii($self,
                                   $self->{'convert_text_options'});
         $basefile = Texinfo::Convert::Text::convert_to_text(
-                                          $command->{'args'}->[0],
+                                          $command->{'contents'}->[0],
                                     $self->{'convert_text_options'});
         Texinfo::Convert::Text::reset_options_code(
                                  $self->{'convert_text_options'});
@@ -929,15 +945,15 @@ sub output_ixin($$)
       } else {
         next;
       }
-      if (defined($command->{'args'}->[4])
-          and $command->{'args'}->[4]->{'contents'}
-          and @{$command->{'args'}->[4]->{'contents'}}) {
+      if (defined($command->{'contents'}->[4])
+          and $command->{'contents'}->[4]->{'contents'}
+          and @{$command->{'contents'}->[4]->{'contents'}}) {
         Texinfo::Convert::Text::set_options_code(
                                  $self->{'convert_text_options'});
         Texinfo::Convert::Text::set_options_encoding_if_not_ascii($self,
                                   $self->{'convert_text_options'});
         $extension = Texinfo::Convert::Text::convert_to_text(
-                                        $command->{'args'}->[4],
+                                        $command->{'contents'}->[4],
                                     $self->{'convert_text_options'});
         Texinfo::Convert::Text::reset_options_code(
                                  $self->{'convert_text_options'});
@@ -958,7 +974,7 @@ sub output_ixin($$)
             $blob_nr++;
             if ($extension eq 'txt') {
               my $encoding
-                 = Texinfo::Common::element_associated_processing_encoding($command);
+                 = Texinfo::Common::associated_processing_encoding($command);
               if (defined($encoding)) {
                 binmode($filehandle, ":encoding($encoding)");
               }
